#include <stdio.h>

#include "bms_bms.h"

#include "bms_define.h"
#include "bms_util.h"
#include "bms_exception.h"


BmsBms::BmsBms( void ) :
headers_(),
wav_array_( "WAV" ),
bmp_array_( "BMP" ),
bpm_array_( "BPM" ),
stop_array_( "STOP" ),
channel_manager_(),
bar_manager_()
{
}

BmsBms::~BmsBms()
{
}

BmsHeaderTable&
BmsBms::GetHeaders( void )
{
  return headers_;
}

BmsRegistArray&
BmsBms::GetWavArray( void )
{
  return wav_array_;
}

BmsRegistArray&
BmsBms::GetBmpArray( void )
{
  return bmp_array_;
}


BmsRegistArray&
BmsBms::GetBpmArray( void )
{
  return bpm_array_;
}


BmsRegistArray&
BmsBms::GetStopArray( void )
{
  return stop_array_;
}


BmsChannelManager&
BmsBms::GetChannelManager( void )
{
  return channel_manager_;
}

BmsBarManager&
BmsBms::GetBarManager( void )
{
  return bar_manager_;
}


int
BmsBms::GetObjectExistsMaxBarPosition( void ) const
{
  int pos = -1;
  int bar = 0;

  for ( BmsChannelManager::ConstIterator m_it = channel_manager_.Begin(); m_it != channel_manager_.End(); ++m_it ) {
    for ( BmsChannel::Iterator c_it = m_it->second->Begin(); c_it != m_it->second->End(); ++c_it ) {
      for ( int i = pos > 0 ? pos : 0; i < static_cast<int>( (*c_it)->GetLength() ); ++i ) {
        if ( (*c_it)->At( i ) != BmsWord::MIN && i >= pos ) {
          for ( ; pos <= i; ++bar ) {
            pos += bar_manager_[bar].GetLength();
          }
          i = pos;
        }
      }
    }
  }
  return bar - 1;
}


void
BmsBms::MultiplyBarDivisionCount( unsigned int multiplier )
{
  bar_manager_.MultiplyBarDivisionCount( multiplier );
  channel_manager_.MultiplyBarDivisionCount( multiplier );
}


namespace {
  void one_work( const BmsRegistArray& from, BmsRegistArray& to, const BmsWord& word ) {
    if ( from.IsExist( word ) ) {
      if ( to.IsExist( word ) ) {
        throw BmsDuplicateHeaderException( from.GetName() + word.ToCharPtr() );
      }
      to.Set( word, from[word] );
    }
  }
}


void
BmsBms::Merge( const BmsBms& other )
{
  for ( BmsHeaderTable::Iterator it = other.headers_.Begin(); it != other.headers_.End(); ++it ) {
    if ( headers_.IsExist( it->first ) ) {
      throw BmsDuplicateHeaderException( it->first );
    }
    headers_.Set( it->first, it->second );
  }
  for ( unsigned int i = 0; i < BmsConst::WORD_MAX_COUNT; ++i ) {
    BmsWord word( i );
    one_work( other.wav_array_,   wav_array_,   word );
    one_work( other.bmp_array_,   bmp_array_,   word );
    one_work( other.bpm_array_,   bpm_array_,   word );
    one_work( other.stop_array_,  stop_array_,  word );
  }

  // TODO
  int lcm = BmsUtil::LCM( bar_manager_.GetBarDivisionCount(), other.bar_manager_.GetBarDivisionCount() );
  if ( lcm > static_cast<int>( bar_manager_.GetBarDivisionCount() ) ) {
    this->MultiplyBarDivisionCount( lcm / bar_manager_.GetBarDivisionCount() );
  }

  for ( unsigned int i = 0; i < BmsConst::BAR_MAX_COUNT; ++i ) {
    if ( other.bar_manager_[i].GetRatio() != 1.0 ) {
      if ( bar_manager_[i].GetRatio() != 1.0 ) {
        throw BmsDuplicateBarChangeException( i );
      }
      bar_manager_[i].SetLength( other.bar_manager_[i].GetLength() * ( lcm / other.bar_manager_.GetBarDivisionCount() ) );
    }
  }

  for ( BmsChannelManager::ConstIterator it = other.channel_manager_.Begin(); it != other.channel_manager_.End(); ++it ) {
    unsigned int i = 0;
    BmsChannel& current_channel = *it->second;
    for ( BmsChannel::ConstIterator current_buffer = current_channel.Begin(); current_buffer != current_channel.End(); ++current_buffer ) {
      if ( bar_manager_.GetBarDivisionCount() == other.bar_manager_.GetBarDivisionCount() ) {
        channel_manager_[current_channel.GetChannelNumber()][i].Merge( **current_buffer );
      }
      else {
        BmsBuffer buf = (*current_buffer)->SubBufferLength( 0, (*current_buffer)->GetLength() );
        channel_manager_[current_channel.GetChannelNumber()][i].Merge( buf );
      }
      ++i;
    }
  }
}


std::string
BmsBms::ToString( void ) const
{
  std::string tmp;

  if ( headers_.GetCount() > 0 ) {
    tmp.append( headers_.ToString() );
    tmp.append( "\n" );
  }
  if ( wav_array_.GetExistCount() > 0 ) {
    tmp.append( wav_array_.ToString() );
    tmp.append( "\n" );
  }
  if ( bmp_array_.GetExistCount() > 0 ) {
    tmp.append( bmp_array_.ToString() );
    tmp.append( "\n" );
  }
  if ( bpm_array_.GetExistCount() > 0 ) {
    tmp.append( bpm_array_.ToString() );
    tmp.append( "\n" );
  }
  if ( stop_array_.GetExistCount() > 0 ) {
    tmp.append( stop_array_.ToString() );
    tmp.append( "\n" );
  }

  int max_bar = this->GetObjectExistsMaxBarPosition();
  unsigned int pos = 0;
  for ( int i = 0; i <= BmsConst::BAR_MAX_VALUE; ++i ) {
    BmsBar current_bar = bar_manager_[i];
    // ߒύXo
    if ( current_bar.GetRatio() != 1.0 ) {
      char buf[1024];
      sprintf( buf, "#%03d%s:%f\n", i, "02", current_bar.GetRatio() );
      tmp.append( buf );
    }
    if ( i > max_bar ) {
      // IuWFĂߒύX͂邩Ȃ̂Ń[v 999 ߂܂ł
      // `l̃IuWFo͂͂܂
      continue;
    }

    for ( BmsChannelManager::ConstIterator it = channel_manager_.Begin(); it != channel_manager_.End(); ++it ) {
      BmsChannel& current_channel = *it->second;
      for ( BmsChannel::ConstIterator current_buffer = current_channel.Begin(); current_buffer != current_channel.End(); ++current_buffer ) {
        int step = current_bar.GetLength();
        for ( unsigned int k = 1; k < current_bar.GetLength(); ++k ) {
          if ( (**current_buffer)[pos + k] != BmsWord::MIN ) {
            step = BmsUtil::GCD( step, k );
          }
        }
        std::string object_array_str;
        for ( unsigned int k = pos; k < pos + current_bar.GetLength(); k += step ) {
          object_array_str.append( (**current_buffer)[k].ToCharPtr() );
        }
        if ( current_channel.GetChannelNumber() == BmsWord( "01" ) ||
             object_array_str.length() > 2 ||
             ( object_array_str.length() == 2 && object_array_str != "00" ) ) {
          char buf[1024];
          sprintf( buf, "#%03d%s:", i, current_channel.GetChannelNumber().ToCharPtr() );
          tmp.append( buf );
          tmp.append( object_array_str );
          tmp.append( "\n" );
        }
      }
    }
    pos += bar_manager_[i].GetLength();
  }
  return tmp;
}
