#ifndef BMSBEL_HANDLER_H_
#define BMSBEL_HANDLER_H_

#include <string>

#include "bms_exception.h"

class BmsParser;

namespace BmsHandler {
  // -- DuplicateHeader --------------------------------------------------
  class DuplicateHeader {
  public:
    static DuplicateHeader DefaultObject;

    class Error {};

  public:
    explicit DuplicateHeader( void );
    virtual ~DuplicateHeader();

    virtual std::string Handle( BmsParser& parser,
                                const std::string header,
                                const std::string before,
                                const std::string present );
  };

  // -- DuplicateBarChange -----------------------------------------------
  class DuplicateBarChange {
  public:
    static DuplicateBarChange DefaultObject;

    class Error {};

  public:
    explicit DuplicateBarChange( void );
    virtual ~DuplicateBarChange();

    virtual double Handle( BmsParser& parser,
                           unsigned int bar,
                           double       before,
                           double       present );
  };

  // -- ParseErrorIgnore -------------------------------------------------
  class ParseErrorIgnore {
  public:
    static ParseErrorIgnore DefaultObject;

  public:
    explicit ParseErrorIgnore( void );
    virtual ~ParseErrorIgnore();

    virtual bool Handle( BmsParser& parser, BmsParseException& e, const std::string& line );
  };

  // -- GenerateRandom ---------------------------------------------------
  class GenerateRandom {
  public:
    virtual ~GenerateRandom( void );
    virtual unsigned int Generate( unsigned int max ) = 0;
    virtual unsigned int GenerateWithParser( BmsParser& parser, unsigned int max );
  };

  // -- GenerateRandomDefault --------------------------------------------
  class GenerateRandomDefault : public GenerateRandom {
  public:
    static GenerateRandomDefault DefaultObject;

  public:
    explicit GenerateRandomDefault( void );
    explicit GenerateRandomDefault( unsigned int seed );

    virtual unsigned int Generate( unsigned int max );

  private:
    unsigned int value_;
  };

  // -- GenerateRandomConstant -------------------------------------------
  class GenerateRandomConstant : public GenerateRandom {
  public:
    explicit GenerateRandomConstant( unsigned int constant );

    virtual unsigned int Generate( unsigned int max );

  private:
    unsigned int constant_;
  };

}

#endif // BMSBEL_HANDLER_H_
