#include <stdio.h>
#include <time.h>

#include "bms_random_manager.h"


// -- BmsRandomElement ---------------------------------------------------
BmsRandomElement::~BmsRandomElement()
{
}


// -- BmsRandomDataElement -----------------------------------------------
BmsRandomDataElement::BmsRandomDataElement( void ) :
data_( *new BmsBms() )
{
}

BmsRandomDataElement::~BmsRandomDataElement()
{
  delete &data_;
}

BmsBms&
BmsRandomDataElement::GetData( void )
{
  return data_;
}


void
BmsRandomDataElement::Resolve( BmsRandomManager& manager, BmsBms& master ) const
{
  NOT_USE_VAR( manager );
  master.Merge( data_ );
}

std::string
BmsRandomDataElement::ToString( void ) const
{
  return data_.ToString();
}


// -- BmsRandomGenerateRandomElement -------------------------------------
BmsRandomGenerateRandomElement::BmsRandomGenerateRandomElement( unsigned int max ) :
max_( max )
{
}

unsigned int
BmsRandomGenerateRandomElement::GetMax( void ) const
{
  return max_;
}

void
BmsRandomGenerateRandomElement::Resolve( BmsRandomManager& manager, BmsBms& master ) const
{
  NOT_USE_VAR( master );
  manager.SetValue( manager.GenerateRandomValue( max_ ) );
}


std::string
BmsRandomGenerateRandomElement::ToString( void ) const
{
  char buf[1024];
  sprintf( buf, "#RANDOM %d\n", max_ );
  return std::string( buf );
}


// -- BmsRandomIfStatementElement ----------------------------------------
BmsRandomIfStatementElement::BmsRandomIfStatementElement( unsigned int prerequisite ) :
prerequisite_( prerequisite ),
children_()
{
}

BmsRandomIfStatementElement::~BmsRandomIfStatementElement()
{
  for ( std::vector<BmsRandomElement*>::iterator it = children_.begin(); it != children_.end(); ++it ) {
    delete *it;
  }
}

unsigned int
BmsRandomIfStatementElement::GetPrequisite( void ) const
{
  return prerequisite_;
}


BmsRandomDataElement&
BmsRandomIfStatementElement::MakeNewDataElement( void )
{
  BmsRandomDataElement* tmp = new BmsRandomDataElement();
  children_.push_back( tmp );
  return *tmp;
}


BmsRandomGenerateRandomElement&
BmsRandomIfStatementElement::MakeNewGenerateRandomElement( unsigned int max )
{
  BmsRandomGenerateRandomElement* tmp = new BmsRandomGenerateRandomElement( max );
  children_.push_back( tmp );
  return *tmp;
}

BmsRandomIfStatementElement&
BmsRandomIfStatementElement::MakeNewIfStatementElement( unsigned int prerequisite )
{
  BmsRandomIfStatementElement* tmp = new BmsRandomIfStatementElement( prerequisite );
  children_.push_back( tmp );
  return *tmp;
}


std::vector<BmsRandomElement*>&
BmsRandomIfStatementElement::GetChildren( void )
{
  return children_;
}


void
BmsRandomIfStatementElement::Resolve( BmsRandomManager& manager, BmsBms& master ) const
{
  if ( manager.GetValue() == prerequisite_ ) {
    for ( std::vector<BmsRandomElement*>::const_iterator it = children_.begin(); it != children_.end(); ++it ) {
      (*it)->Resolve( manager, master );
    }
  }
}


std::string
BmsRandomIfStatementElement::ToString( void ) const
{
  std::string tmp;
  if ( prerequisite_ != 0 ) {
    char buf[1024];
    sprintf( buf, "#IF %d\n\n", prerequisite_ );
    tmp.append( buf );
  }
  for ( std::vector<BmsRandomElement*>::const_iterator it = children_.begin(); it != children_.end(); ++it ) {
    tmp.append( (*it)->ToString() );
    tmp.append( "\n" );
  }
  if ( prerequisite_ != 0 ) {
    tmp.append( "#ENDIF\n\n" );
  }
  return tmp;
}


// -- BmsRandomManager ---------------------------------------------------
BmsRandomManager::BmsRandomManager( BmsHandler::GenerateRandom& generate_random_handler ) :
root_( 0 ),
root_data_( root_.MakeNewDataElement() ),
value_( 0 ),
generate_random_handler_( generate_random_handler )
{
}


BmsRandomIfStatementElement&
BmsRandomManager::GetRoot( void )
{
  return root_;
}

BmsRandomDataElement&
BmsRandomManager::GetRootData( void )
{
  return root_data_;
}


unsigned int
BmsRandomManager::GetValue( void ) const
{
  return value_;
}

void
BmsRandomManager::SetValue( unsigned int value )
{
  value_ = value;
}

unsigned int
BmsRandomManager::GenerateRandomValue( unsigned int max ) const
{
  return generate_random_handler_.Generate( max );
}

void
BmsRandomManager::Resolve( BmsBms& master )
{
  root_.Resolve( *this, master );
}


std::string
BmsRandomManager::ToString( void ) const
{
  return root_.ToString();
}
